==> hiawatha-3.7.tar.gz: hiawatha-3.7/mimetype.c <==
#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include "libstr.h"
#include "mimetype.h"

#define MAX_EXT 6

char *default_mimetype = "text/plain";

/* Read a mimetype configuration file.
 */
t_mimetype *read_mimetypes(char *configfile) {
	FILE *fp;
	char *line, *data, *next, *ext;
	bool quit;
	t_mimetype *mime;
	t_mimename *w_mime = NULL;
	t_extension *w_ext = NULL;

	/* Read mimetype configurationfile.
	 */
	if ((fp = fopen(configfile, "r")) == NULL) {
		return NULL;
	}
	if ((mime = (t_mimetype*)malloc(sizeof(t_mimetype))) == NULL) {
		fclose(fp);
		return NULL;
	}
	if ((line = (char*)malloc(257)) == NULL) {
		fclose(fp);
		free(mime);
		return NULL;
	}
	*(line + 256) = '\0';

	while (fgets(line, 256, fp) != NULL) {
		next = data = strdup(line);
		while (*next != '\0') {
			if (*next == '\t') {
				*next = ' ';
			} else if (*next == '\n') {
				*next = '\0';
			}
			next++;
		}
		if ((next = strchr(data, ' ')) != NULL) {
			*next++ = '\0';
			if (w_mime == NULL) {
				mime->mimename = w_mime = (t_mimename*)malloc(sizeof(t_mimename));
			} else {
				w_mime->next = (t_mimename*)malloc(sizeof(t_mimename));
				w_mime = w_mime->next;
			}
			w_mime->next = NULL;
			w_mime->name = data;

			quit = false;
			while (quit == false) {
				while (*next == ' ') {
					next++;
				}
				if (*next != '\0') {
					ext = next;
					if ((next = strchr(next, ' ')) != NULL) {
						*next++ = '\0';
					} else {
						quit = true;
					}
					if (strlen(ext) <= MAX_EXT) {
						if (w_ext == NULL) {
							mime->extension = w_ext = (t_extension*)malloc(sizeof(t_extension));
						} else {
							w_ext->next = (t_extension*)malloc(sizeof(t_extension));
							w_ext = w_ext->next;
						}
						w_ext->next = NULL;
						w_ext->mimename = w_mime;
						w_ext->name = ext;
						strlower(w_ext->name);
					}
				} else {
					quit = true;
				}
			}
		}
	}

	free(line);
	fclose(fp);

	return mime;
}

/* Return the mimetype of a file.
 */
char *get_mimetype(char *filename, t_mimetype *mime) {
	char *ext, *ext_lower, *result = default_mimetype;
	t_extension *w_ext = NULL;

	/* Convert file extension to mimetype.
	 */ 
	if (filename != NULL) {
		if ((ext = strrchr(filename, '.')) != NULL) {
			if ((ext_lower = strlower(strdup(++ext))) != NULL) {
				w_ext = mime->extension;
				while (w_ext != NULL) {
					if (strcmp(ext_lower, w_ext->name) == 0) {
						result = w_ext->mimename->name;
						break;
					}
					w_ext = w_ext->next;
				}
				free(ext_lower);
			}
		}
	}

	return result;
}

/* Remove a mimetype list.
 */
t_mimetype *remove_mimetypes(t_mimetype *mime) {
	t_mimename *type;

	if (mime != NULL) {
		while (mime->mimename != NULL) {
			type = mime->mimename;
			mime->mimename = mime->mimename->next;
			free(type);
		}
		free(mime);
	}

	return NULL;
}

==> hiawatha-4.3.2.tar.gz: hiawatha-4.3.2/mimetype.c <==
#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <string.h>
#include "libstr.h"
#include "mimetype.h"

#define MAX_EXT 6

char *default_mimetype = "text/plain";

/* Read a mimetype configuration file.
 */
int read_mimetypes(char *configfile, t_mimetype **mime) {
	FILE *fp;
	char *line, *data, *next, *ext;
	bool quit;
	t_mimename *w_mime = NULL;
	t_extension *w_ext = NULL;

	/* Read mimetype configurationfile.
	 */
	if ((fp = fopen(configfile, "r")) == NULL) {
		return -1;
	}
	if ((*mime = (t_mimetype*)malloc(sizeof(t_mimetype))) == NULL) {
		fclose(fp);
		return -1;
	}
	if ((line = (char*)malloc(257)) == NULL) {
		fclose(fp);
		free(*mime);
		return -1;
	}
	*(line + 256) = '\0';

	while (fgets(line, 256, fp) != NULL) {
		next = data = strdup(line);
		while (*next != '\0') {
			if (*next == '\t') {
				*next = ' ';
			} else if (*next == '\n') {
				*next = '\0';
			}
			next++;
		}
		if ((next = strchr(data, ' ')) != NULL) {
			*next++ = '\0';
			if (w_mime == NULL) {
				(*mime)->mimename = w_mime = (t_mimename*)malloc(sizeof(t_mimename));
			} else {
				w_mime->next = (t_mimename*)malloc(sizeof(t_mimename));
				w_mime = w_mime->next;
			}
			w_mime->next = NULL;
			w_mime->name = data;

			quit = false;
			while (quit == false) {
				while (*next == ' ') {
					next++;
				}
				if (*next != '\0') {
					ext = next;
					if ((next = strchr(next, ' ')) != NULL) {
						*next++ = '\0';
					} else {
						quit = true;
					}
					if (strlen(ext) <= MAX_EXT) {
						if (w_ext == NULL) {
							(*mime)->extension = w_ext = (t_extension*)malloc(sizeof(t_extension));
						} else {
							w_ext->next = (t_extension*)malloc(sizeof(t_extension));
							w_ext = w_ext->next;
						}
						w_ext->next = NULL;
						w_ext->mimename = w_mime;
						w_ext->name = ext;
						strlower(w_ext->name);
					}
				} else {
					quit = true;
				}
			}
		}
	}

	free(line);
	fclose(fp);

	return 0;
}

/* Return the mimetype of a file.
 */
char *get_mimetype(char *filename, t_mimetype *mime) {
	char *ext, *ext_lower, *result = default_mimetype;
	t_extension *w_ext = NULL;

	/* Convert file extension to mimetype.
	 */ 
	if (filename != NULL) {
		if ((ext = strrchr(filename, '.')) != NULL) {
			if ((ext_lower = strlower(strdup(++ext))) != NULL) {
				w_ext = mime->extension;
				while (w_ext != NULL) {
					if (strcmp(ext_lower, w_ext->name) == 0) {
						result = w_ext->mimename->name;
						break;
					}
					w_ext = w_ext->next;
				}
				free(ext_lower);
			}
		}
	}

	return result;
}
